/*==============================================================================*/
/*  Copyright (C) 2020, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   GPIO 4bit Driver                                                 */
/*  EventID:   Via the driver                                                   */
/*  AppVer :   DT+ Ver1.00`                                                    */
/*  FileID :   33c                                                              */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for File IO                                                  */
/*==============================================================================*/
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT+ Project setting.     */
/*==============================================================================*/
#define	DT_UINT unsigned int


/*==============================================================================*/
/*  Struct:  DT_DATA                                                            */
/*  Desc:    Test Point Data Struct                                             */
/*==============================================================================*/
struct DT_DATA {
	char tp_func;
	DT_UINT addr;
	DT_UINT dat;
	DT_UINT size;
	unsigned char value[256];
};

#define TP_BusOut		'2'
#define TP_MemoryOutput		'3'

/*==============================================================================*/
/*  Macro:  PROC_NAME                                                           */
/*  Desc:   Register Proc File name for Linux.                                  */
/*==============================================================================*/
#define DT_DRV	"/proc/dtproc"

static int dat_file = 0;
static int BusoutMode = 0;
/*==============================================================================*/
/*  Macro:  BusOutModeSet                                                       */
/*  Desc:   0 => open file 1 => writing 2 => close file                         */
/*==============================================================================*/
#define BusOutModeSet(mode) BusoutMode = mode

/*==============================================================================*/
/*  Func:   _TP_BusOut                                                          */
/*  Desc:   Called by Test Point                                                */
/*==============================================================================*/
void _TP_BusOut(DT_UINT addr, DT_UINT dat)
{
	struct DT_DATA dt_data;
	
	if (BusoutMode == 0) {
		dat_file = open(DT_DRV, O_RDWR);
		if (dat_file <= 0) {
			printf("[DT] %s open error \n", DT_DRV);
			return;
		}
		printf("[DT] %s open ok \n", DT_DRV);
		BusOutModeSet(1);
	}

	dt_data.tp_func = TP_BusOut;
	dt_data.addr = addr;
	dt_data.dat = dat;
	
	write(dat_file, &dt_data, sizeof(dt_data));
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable TestPoint                                        */
/*==============================================================================*/
void _TP_MemoryOutput(DT_UINT addr, DT_UINT dat, void *value, DT_UINT size)
{
	struct DT_DATA dt_data;

	if (BusoutMode == 0) {
		dat_file = open(DT_DRV, O_RDWR);
		if (dat_file <= 0) {
			printf("[DT] %s open error \n", DT_DRV);
			return;
		}
		BusOutModeSet(1);
	}
	dt_data.tp_func = TP_MemoryOutput;
	dt_data.addr = addr;
	dt_data.dat = dat;
	dt_data.size = size;
	memcpy(dt_data.value, value, size);
	
	write(dat_file, &dt_data, sizeof(dt_data));
}


